@@ -34,8 +34,9 @@
 #include <dbus/dbus-spawn.h>
 #include <dbus/dbus-timeout.h>
 #include <dbus/dbus-sysdeps.h>
-#include <dirent.h>
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
 
 #define DBUS_SERVICE_SECTION "D-BUS Service"
 #define DBUS_SERVICE_NAME "Name"
@@ -736,9 +737,15 @@ bus_activation_new (BusContext        *context,
           goto failed;
         }
 
+      /* only fail on OOM, it is ok if we can't read the directory */
       if (!update_directory (activation, s_dir, error))
-        goto failed;
-      
+        { 
+          if (dbus_error_has_name (error, DBUS_ERROR_NO_MEMORY)) 
+            goto failed;
+          else
+            dbus_error_free (error);
+        }
+
       link = _dbus_list_get_next_link (directories, link);
     }
 
@@ -1304,6 +1311,7 @@ bus_activation_activate_service (BusActivation  *activation,
   DBusMessage *message;
   DBusString service_str;
   char **argv;
+  char **envp = NULL;
   int argc;
   dbus_bool_t retval;
   DBusHashIter iter;
@@ -1535,6 +1543,7 @@ bus_activation_activate_service (BusActivation  *activation,
 
   _dbus_verbose ("Spawning %s ...\n", argv[0]);
   if (!_dbus_spawn_async_with_babysitter (&pending_activation->babysitter, argv,
+                                          envp,
                                           child_setup, activation, 
                                           error))
     {
