@@ -38,6 +38,7 @@
 struct BusContext
 {
   int refcount;
+  DBusGUID uuid;
   char *config_file;
   char *type;
   char *address;
@@ -50,7 +51,6 @@ struct BusContext
   BusRegistry *registry;
   BusPolicy *policy;
   BusMatchmaker *matchmaker;
-  DBusUserDatabase *user_database;
   BusLimits limits;
   unsigned int fork : 1;
 };
@@ -402,6 +402,7 @@ process_config_every_time (BusContext      *context,
 {
   DBusString full_address;
   DBusList *link;
+  BusActivation *new_activation;
   char *addr;
 
   dbus_bool_t retval;
@@ -467,19 +468,20 @@ process_config_every_time (BusContext      *context,
     }
 
   /* Create activation subsystem */
-  
-  if (is_reload)
-    bus_activation_unref (context->activation);
-  
-  context->activation = bus_activation_new (context, &full_address,
-                                            bus_config_parser_get_service_dirs (parser),
-                                            error);
-  if (context->activation == NULL)
+  new_activation = bus_activation_new (context, &full_address,
+				       bus_config_parser_get_service_dirs (parser),
+				       error);
+  if (new_activation == NULL)
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
 
+  if (is_reload)
+    bus_activation_unref (context->activation);
+
+  context->activation = new_activation;
+
   /* Drop existing conf-dir watches (if applicable) */
 
   if (is_reload)
@@ -517,7 +519,7 @@ process_config_postinit (BusContext      *context,
   /* Watch all conf directories */
   _dbus_list_foreach (bus_config_parser_get_conf_dirs (parser),
 		      (DBusForeachFunction) bus_watch_directory,
-		      NULL);
+		      context);
 
   return TRUE;
 }
@@ -525,13 +527,12 @@ process_config_postinit (BusContext      *context,
 BusContext*
 bus_context_new (const DBusString *config_file,
                  ForceForkSetting  force_fork,
-                 int               print_addr_fd,
-                 int               print_pid_fd,
+                 DBusPipe         *print_addr_pipe,
+                 DBusPipe         *print_pid_pipe,
                  DBusError        *error)
 {
   BusContext *context;
   BusConfigParser *parser;
-  DBusCredentials creds;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -552,6 +553,8 @@ bus_context_new (const DBusString *config_file,
     }
   context->refcount = 1;
 
+  _dbus_generate_uuid (&context->uuid);
+  
   if (!_dbus_string_copy_data (config_file, &context->config_file))
     {
       BUS_SET_OOM (error);
@@ -596,19 +599,12 @@ bus_context_new (const DBusString *config_file,
   if (!dbus_server_allocate_data_slot (&server_data_slot))
     _dbus_assert_not_reached ("second ref of server data slot failed");
 
-  context->user_database = _dbus_user_database_new ();
-  if (context->user_database == NULL)
-    {
-      BUS_SET_OOM (error);
-      goto failed;
-    }
-  
-  /* Note that we don't know whether the print_addr_fd is
+  /* Note that we don't know whether the print_addr_pipe is
    * one of the sockets we're using to listen on, or some
    * other random thing. But I think the answer is "don't do
    * that then"
    */
-  if (print_addr_fd >= 0)
+  if (print_addr_pipe != NULL && _dbus_pipe_is_valid (print_addr_pipe))
     {
       DBusString addr;
       const char *a = bus_context_get_address (context);
@@ -630,17 +626,20 @@ bus_context_new (const DBusString *config_file,
         }
 
       bytes = _dbus_string_get_length (&addr);
-      if (_dbus_write_socket (print_addr_fd, &addr, 0, bytes) != bytes)
+      if (_dbus_pipe_write (print_addr_pipe, &addr, 0, bytes, error) != bytes)
         {
-          dbus_set_error (error, DBUS_ERROR_FAILED,
-                          "Printing message bus address: %s\n",
-                          _dbus_strerror (errno));
+          /* pipe write returns an error on failure but not short write */
+          if (error != NULL && !dbus_error_is_set (error))
+            {
+              dbus_set_error (error, DBUS_ERROR_FAILED,
+                              "Printing message bus address: did not write all bytes\n");
+            }
           _dbus_string_free (&addr);
           goto failed;
         }
 
-      if (print_addr_fd > 2)
-        _dbus_close_socket (print_addr_fd, NULL);
+      if (!_dbus_pipe_is_stdout_or_stderr (print_addr_pipe))
+        _dbus_pipe_close (print_addr_pipe, NULL);
 
       _dbus_string_free (&addr);
     }
@@ -662,13 +661,7 @@ bus_context_new (const DBusString *config_file,
   /* check user before we fork */
   if (context->user != NULL)
     {
-      DBusString u;
-
-      _dbus_string_init_const (&u, context->user);
-
-      if (!_dbus_credentials_from_username (&u, &creds) ||
-          creds.uid < 0 ||
-          creds.gid < 0)
+      if (!_dbus_verify_daemon_user (context->user))
         {
           dbus_set_error (error, DBUS_ERROR_FAILED,
                           "Could not get UID and GID for username \"%s\"",
@@ -686,7 +679,7 @@ bus_context_new (const DBusString *config_file,
         _dbus_string_init_const (&u, context->pidfile);
       
       if (!_dbus_become_daemon (context->pidfile ? &u : NULL, 
-				print_pid_fd,
+				print_pid_pipe,
 				error))
 	{
 	  _DBUS_ASSERT_ERROR_IS_SET (error);
@@ -711,7 +704,7 @@ bus_context_new (const DBusString *config_file,
     }
 
   /* Write PID if requested */
-  if (print_pid_fd >= 0)
+  if (print_pid_pipe != NULL && _dbus_pipe_is_valid (print_pid_pipe))
     {
       DBusString pid;
       int bytes;
@@ -731,26 +724,24 @@ bus_context_new (const DBusString *config_file,
         }
 
       bytes = _dbus_string_get_length (&pid);
-      if (_dbus_write_socket (print_pid_fd, &pid, 0, bytes) != bytes)
+      if (_dbus_pipe_write (print_pid_pipe, &pid, 0, bytes, error) != bytes)
         {
-          dbus_set_error (error, DBUS_ERROR_FAILED,
-                          "Printing message bus PID: %s\n",
-                          _dbus_strerror (errno));
+          /* pipe_write sets error on failure but not short write */
+          if (error != NULL && !dbus_error_is_set (error))
+            {
+              dbus_set_error (error, DBUS_ERROR_FAILED,
+                              "Printing message bus PID: did not write enough bytes\n");
+            }
           _dbus_string_free (&pid);
           goto failed;
         }
 
-      if (print_pid_fd > 2)
-        _dbus_close_socket (print_pid_fd, NULL);
+      if (!_dbus_pipe_is_stdout_or_stderr (print_pid_pipe))
+        _dbus_pipe_close (print_pid_pipe, NULL);
       
       _dbus_string_free (&pid);
     }
 
-  if (!bus_selinux_full_init ())
-    {
-      _dbus_warn ("SELinux initialization failed\n");
-    }
-
   if (!process_config_postinit (context, parser, error))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
@@ -768,12 +759,17 @@ bus_context_new (const DBusString *config_file,
    */
   if (context->user != NULL)
     {
-      if (!_dbus_change_identity (creds.uid, creds.gid, error))
+      if (!_dbus_change_to_daemon_user (context->user, error))
 	{
 	  _DBUS_ASSERT_ERROR_IS_SET (error);
 	  goto failed;
 	}
     }
+
+  if (!bus_selinux_full_init ())
+    {
+      _dbus_warn ("SELinux initialization failed\n");
+    }
   
   dbus_server_free_data_slot (&server_data_slot);
   
@@ -791,6 +787,13 @@ bus_context_new (const DBusString *config_file,
   return NULL;
 }
 
+dbus_bool_t
+bus_context_get_id (BusContext       *context,
+                    DBusString       *uuid)
+{
+  return _dbus_uuid_encode (&context->uuid, uuid);
+}
+
 dbus_bool_t
 bus_context_reload_config (BusContext *context,
 			   DBusError  *error)
@@ -800,7 +803,7 @@ bus_context_reload_config (BusContext *context,
   dbus_bool_t ret;
 
   /* Flush the user database cache */
-  _dbus_user_database_flush(context->user_database);
+  _dbus_flush_caches ();
 
   ret = FALSE;
   _dbus_string_init_const (&config_file, context->config_file);
@@ -951,10 +954,6 @@ bus_context_unref (BusContext *context)
 
           dbus_free (context->pidfile); 
 	}
-
-      if (context->user_database != NULL)
-	_dbus_user_database_unref (context->user_database);
-      
       dbus_free (context);
 
       dbus_server_free_data_slot (&server_data_slot);
@@ -1004,19 +1003,24 @@ bus_context_get_loop (BusContext *context)
   return context->loop;
 }
 
-DBusUserDatabase*
-bus_context_get_user_database (BusContext *context)
+dbus_bool_t
+bus_context_allow_unix_user (BusContext   *context,
+                             unsigned long uid)
 {
-  return context->user_database;
+  return bus_policy_allow_unix_user (context->policy,
+                                     uid);
 }
 
+/* For now this is never actually called because the default
+ * DBusConnection behavior of 'same user that owns the bus can connect'
+ * is all it would do.
+ */
 dbus_bool_t
-bus_context_allow_user (BusContext   *context,
-                        unsigned long uid)
+bus_context_allow_windows_user (BusContext       *context,
+                                const char       *windows_sid)
 {
-  return bus_policy_allow_user (context->policy,
-                                context->user_database,
-                                uid);
+  return bus_policy_allow_windows_user (context->policy,
+                                        windows_sid);
 }
 
 BusPolicy *
