@@ -460,8 +460,8 @@ bus_config_parser_new (const DBusString      *basedir,
     {
 
       /* Make up some numbers! woot! */
-      parser->limits.max_incoming_bytes = _DBUS_ONE_MEGABYTE * 63;
-      parser->limits.max_outgoing_bytes = _DBUS_ONE_MEGABYTE * 63;
+      parser->limits.max_incoming_bytes = _DBUS_ONE_MEGABYTE * 127;
+      parser->limits.max_outgoing_bytes = _DBUS_ONE_MEGABYTE * 127;
       parser->limits.max_message_size = _DBUS_ONE_MEGABYTE * 32;
       
       /* Making this long means the user has to wait longer for an error
@@ -476,22 +476,26 @@ bus_config_parser_new (const DBusString      *basedir,
        */
       parser->limits.auth_timeout = 30000; /* 30 seconds */
       
-      parser->limits.max_incomplete_connections = 32;
-      parser->limits.max_connections_per_user = 128;
+      parser->limits.max_incomplete_connections = 64;
+      parser->limits.max_connections_per_user = 256;
       
       /* Note that max_completed_connections / max_connections_per_user
        * is the number of users that would have to work together to
        * DOS all the other users.
        */
-      parser->limits.max_completed_connections = 1024;
+      parser->limits.max_completed_connections = 2048;
       
-      parser->limits.max_pending_activations = 256;
-      parser->limits.max_services_per_connection = 256;
+      parser->limits.max_pending_activations = 512;
+      parser->limits.max_services_per_connection = 512;
       
       parser->limits.max_match_rules_per_connection = 512;
       
       parser->limits.reply_timeout = 5 * 60 * 1000; /* 5 minutes */
-      parser->limits.max_replies_per_connection = 32;
+
+      /* this is effectively a limit on message queue size for messages
+       * that require a reply
+       */
+      parser->limits.max_replies_per_connection = 1024*8;
     }
       
   parser->refcount = 1;
@@ -978,8 +982,8 @@ start_busconfig_child (BusConfigParser   *parser,
           DBusString username;
           _dbus_string_init_const (&username, user);
 
-          if (_dbus_get_user_id (&username,
-                                 &e->d.policy.gid_uid_or_at_console))
+          if (_dbus_parse_unix_user_from_config (&username,
+                                                 &e->d.policy.gid_uid_or_at_console))
             e->d.policy.type = POLICY_USER;
           else
             _dbus_warn ("Unknown username \"%s\" in message bus configuration file\n",
@@ -990,8 +994,8 @@ start_busconfig_child (BusConfigParser   *parser,
           DBusString group_name;
           _dbus_string_init_const (&group_name, group);
 
-          if (_dbus_get_group_id (&group_name,
-                                  &e->d.policy.gid_uid_or_at_console))
+          if (_dbus_parse_unix_group_from_config (&group_name,
+                                                  &e->d.policy.gid_uid_or_at_console))
             e->d.policy.type = POLICY_GROUP;
           else
             _dbus_warn ("Unknown group \"%s\" in message bus configuration file\n",
@@ -1176,7 +1180,6 @@ append_rule_from_element (BusConfigParser   *parser,
        (send_interface && receive_member) ||
        (send_interface && receive_error) ||
        (send_interface && receive_sender) ||
-       (send_interface && eavesdrop) ||
        (send_interface && receive_requested_reply) ||
        (send_interface && own) ||
        (send_interface && user) ||
@@ -1187,7 +1190,6 @@ append_rule_from_element (BusConfigParser   *parser,
        (send_member && receive_member) ||
        (send_member && receive_error) ||
        (send_member && receive_sender) ||
-       (send_member && eavesdrop) ||
        (send_member && receive_requested_reply) ||
        (send_member && own) ||
        (send_member && user) ||
@@ -1197,7 +1199,6 @@ append_rule_from_element (BusConfigParser   *parser,
        (send_error && receive_member) ||
        (send_error && receive_error) ||
        (send_error && receive_sender) ||
-       (send_error && eavesdrop) ||
        (send_error && receive_requested_reply) ||
        (send_error && own) ||
        (send_error && user) ||
@@ -1207,7 +1208,6 @@ append_rule_from_element (BusConfigParser   *parser,
        (send_destination && receive_member) ||
        (send_destination && receive_error) ||
        (send_destination && receive_sender) ||
-       (send_destination && eavesdrop) ||
        (send_destination && receive_requested_reply) ||
        (send_destination && own) ||
        (send_destination && user) ||
@@ -1217,7 +1217,6 @@ append_rule_from_element (BusConfigParser   *parser,
        (send_type && receive_member) ||
        (send_type && receive_error) ||
        (send_type && receive_sender) ||
-       (send_type && eavesdrop) ||
        (send_type && receive_requested_reply) ||
        (send_type && own) ||
        (send_type && user) ||
@@ -1227,7 +1226,6 @@ append_rule_from_element (BusConfigParser   *parser,
        (send_path && receive_member) ||
        (send_path && receive_error) ||
        (send_path && receive_sender) ||
-       (send_path && eavesdrop) ||
        (send_path && receive_requested_reply) ||
        (send_path && own) ||
        (send_path && user) ||
@@ -1237,7 +1235,6 @@ append_rule_from_element (BusConfigParser   *parser,
        (send_requested_reply && receive_member) ||
        (send_requested_reply && receive_error) ||
        (send_requested_reply && receive_sender) ||
-       (send_requested_reply && eavesdrop) ||
        (send_requested_reply && receive_requested_reply) ||
        (send_requested_reply && own) ||
        (send_requested_reply && user) ||
@@ -1314,6 +1311,16 @@ append_rule_from_element (BusConfigParser   *parser,
             }
         }
 
+      if (eavesdrop &&
+          !(strcmp (eavesdrop, "true") == 0 ||
+            strcmp (eavesdrop, "false") == 0))
+        {
+          dbus_set_error (error, DBUS_ERROR_FAILED,
+                          "Bad value \"%s\" for %s attribute, must be true or false",
+                          "eavesdrop", eavesdrop);
+          return FALSE;
+        }
+
       if (send_requested_reply &&
           !(strcmp (send_requested_reply, "true") == 0 ||
             strcmp (send_requested_reply, "false") == 0))
@@ -1328,9 +1335,12 @@ append_rule_from_element (BusConfigParser   *parser,
       if (rule == NULL)
         goto nomem;
       
+      if (eavesdrop)
+        rule->d.send.eavesdrop = (strcmp (eavesdrop, "true") == 0);
+
       if (send_requested_reply)
         rule->d.send.requested_reply = (strcmp (send_requested_reply, "true") == 0);
-      
+
       rule->d.send.message_type = message_type;
       rule->d.send.path = _dbus_strdup (send_path);
       rule->d.send.interface = _dbus_strdup (send_interface);
@@ -1458,7 +1468,7 @@ append_rule_from_element (BusConfigParser   *parser,
           
           _dbus_string_init_const (&username, user);
       
-          if (_dbus_get_user_id (&username, &uid))
+          if (_dbus_parse_unix_user_from_config (&username, &uid))
             {
               rule = bus_policy_rule_new (BUS_POLICY_RULE_USER, allow); 
               if (rule == NULL)
@@ -1490,7 +1500,7 @@ append_rule_from_element (BusConfigParser   *parser,
           
           _dbus_string_init_const (&groupname, group);
           
-          if (_dbus_get_user_id (&groupname, &gid))
+          if (_dbus_parse_unix_group_from_config (&groupname, &gid))
             {
               rule = bus_policy_rule_new (BUS_POLICY_RULE_GROUP, allow); 
               if (rule == NULL)
@@ -1560,7 +1570,7 @@ append_rule_from_element (BusConfigParser   *parser,
 
         case POLICY_CONSOLE:
           if (!bus_policy_append_console_rule (parser->policy, pe->d.policy.gid_uid_or_at_console,
-                                             rule))
+                                               rule))
             goto nomem;
           break;
         }
@@ -3066,10 +3076,14 @@ process_test_equiv_subdir (const DBusString *test_base_dir,
 
 static const char *test_service_dir_matches[] = 
         {
+#ifdef DBUS_UNIX
          "/testusr/testlocal/testshare/dbus-1/services",
          "/testusr/testshare/dbus-1/services",
+#endif
          DBUS_DATADIR"/dbus-1/services",
-         "/testhome/foo/.testlocal/testshare/dbus-1/services",         
+#ifdef DBUS_UNIX
+         "/testhome/foo/.testlocal/testshare/dbus-1/services",
+#endif
          NULL
         };
 
@@ -3078,11 +3092,36 @@ test_default_session_servicedirs (void)
 {
   DBusList *dirs;
   DBusList *link;
+  DBusString progs;
+  const char *common_progs;
   int i;
 
+  /* On Unix we don't actually use this variable, but it's easier to handle the
+   * deallocation if we always allocate it, whether needed or not */
+  if (!_dbus_string_init (&progs))
+    _dbus_assert_not_reached ("OOM allocating progs");
+
+  common_progs = _dbus_getenv ("CommonProgramFiles");
+#ifndef DBUS_UNIX
+  if (common_progs) 
+    {
+      if (!_dbus_string_append (&progs, common_progs)) 
+        {
+          _dbus_string_free (&progs);
+          return FALSE;
+        }
+
+      if (!_dbus_string_append (&progs, "/dbus-1/services")) 
+        {
+          _dbus_string_free (&progs);
+          return FALSE;
+        }
+      test_service_dir_matches[1] = _dbus_string_get_const_data(&progs);
+    }
+#endif
   dirs = NULL;
 
-  printf ("Testing retriving the default session service directories\n");
+  printf ("Testing retrieving the default session service directories\n");
   if (!_dbus_get_standard_session_servicedirs (&dirs))
     _dbus_assert_not_reached ("couldn't get stardard dirs");
 
@@ -3093,9 +3132,12 @@ test_default_session_servicedirs (void)
       
       printf ("    default service dir: %s\n", (char *)link->data);
       _dbus_string_init_const (&path, (char *)link->data);
-      if (!_dbus_string_ends_with_c_str (&path, "share/dbus-1/services"))
+      if (!_dbus_string_ends_with_c_str (&path, "dbus-1/services"))
         {
           printf ("error with default session service directories\n");
+	      dbus_free (link->data);
+    	  _dbus_list_free_link (link);
+          _dbus_string_free (&progs);
           return FALSE;
         }
  
@@ -3103,12 +3145,13 @@ test_default_session_servicedirs (void)
       _dbus_list_free_link (link);
     }
 
+#ifdef DBUS_UNIX
   if (!_dbus_setenv ("XDG_DATA_HOME", "/testhome/foo/.testlocal/testshare"))
     _dbus_assert_not_reached ("couldn't setenv XDG_DATA_HOME");
 
   if (!_dbus_setenv ("XDG_DATA_DIRS", ":/testusr/testlocal/testshare: :/testusr/testshare:"))
     _dbus_assert_not_reached ("couldn't setenv XDG_DATA_DIRS");
-
+#endif
   if (!_dbus_get_standard_session_servicedirs (&dirs))
     _dbus_assert_not_reached ("couldn't get stardard dirs");
 
@@ -3120,6 +3163,9 @@ test_default_session_servicedirs (void)
       if (test_service_dir_matches[i] == NULL)
         {
           printf ("more directories parsed than in match set\n");
+          dbus_free (link->data);
+          _dbus_list_free_link (link);
+          _dbus_string_free (&progs);
           return FALSE;
         }
  
@@ -3129,6 +3175,9 @@ test_default_session_servicedirs (void)
           printf ("%s directory does not match %s in the match set\n", 
                   (char *)link->data,
                   test_service_dir_matches[i]);
+          dbus_free (link->data);
+          _dbus_list_free_link (link);
+          _dbus_string_free (&progs);
           return FALSE;
         }
 
@@ -3143,9 +3192,11 @@ test_default_session_servicedirs (void)
       printf ("extra data %s in the match set was not matched\n",
               test_service_dir_matches[i]);
 
+      _dbus_string_free (&progs);
       return FALSE;
     }
     
+  _dbus_string_free (&progs);
   return TRUE;
 }
 			   
