@@ -63,6 +63,7 @@ struct BusPolicyRule
       char *member;
       char *error;
       char *destination;
+      unsigned int eavesdrop : 1;
       unsigned int requested_reply : 1;
     } send;
 
@@ -112,9 +113,10 @@ void             bus_policy_unref                 (BusPolicy        *policy);
 BusClientPolicy* bus_policy_create_client_policy  (BusPolicy        *policy,
                                                    DBusConnection   *connection,
                                                    DBusError        *error);
-dbus_bool_t      bus_policy_allow_user            (BusPolicy        *policy,
-                                                   DBusUserDatabase *user_database,
+dbus_bool_t      bus_policy_allow_unix_user       (BusPolicy        *policy,
                                                    unsigned long     uid);
+dbus_bool_t      bus_policy_allow_windows_user    (BusPolicy        *policy,
+                                                   const char       *windows_sid);
 dbus_bool_t      bus_policy_append_default_rule   (BusPolicy        *policy,
                                                    BusPolicyRule    *rule);
 dbus_bool_t      bus_policy_append_mandatory_rule (BusPolicy        *policy,
