@@ -50,6 +50,14 @@
 #  endif /* !__cplusplus */
 #endif
 
+#if  __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1)
+#  define DBUS_DEPRECATED __attribute__ ((__deprecated__))
+#elif defined(_MSC_VER) && (_MSC_VER >= 1300)
+#  define DBUS_DEPRECATED __declspec(deprecated)
+#else
+#  define DBUS_DEPRECATED
+#endif
+
 /* Normally docs are in .c files, but there isn't a .c file for this. */
 /**
  * @defgroup DBusMacros Utility macros
@@ -67,6 +75,9 @@
  * Macro used prior to declaring functions in the D-Bus header
  * files. Expands to "extern "C"" when using a C++ compiler,
  * and expands to nothing when using a C compiler.
+ *
+ * Please don't use this in your own code, consider it
+ * D-Bus internal.
  */
 /**
  * @def DBUS_END_DECLS
@@ -74,6 +85,9 @@
  * Macro used after declaring functions in the D-Bus header
  * files. Expands to "}" when using a C++ compiler,
  * and expands to nothing when using a C compiler.
+ *
+ * Please don't use this in your own code, consider it
+ * D-Bus internal.
  */
 /**
  * @def TRUE
@@ -90,6 +104,20 @@
  *
  * A null pointer, defined appropriately for C or C++.
  */
+/**
+ * @def DBUS_DEPRECATED
+ *
+ * Tells the compiler to warn about a function or type if it's used.
+ * Code marked in this way should also be enclosed in
+ * @code
+ * #ifndef DBUS_DISABLE_DEPRECATED
+ *  deprecated stuff here
+ * #endif
+ * @endcode
+ *
+ * Please don't use this in your own code, consider it
+ * D-Bus internal.
+ */
 
 /** @} */
 
