@@ -31,6 +31,8 @@
 #include "dbus-string.h"
 #include "dbus-userdb.h"
 #include "dbus-list.h"
+#include "dbus-credentials.h"
+
 #include <sys/types.h>
 #include <stdlib.h>
 #include <string.h>
@@ -76,15 +78,16 @@
 #endif
 
 static dbus_bool_t
-_dbus_open_socket (int              *fd,
+_dbus_open_socket (int              *fd_p,
                    int               domain,
                    int               type,
                    int               protocol,
                    DBusError        *error)
 {
-  *fd = socket (domain, type, protocol);
-  if (fd >= 0)
+  *fd_p = socket (domain, type, protocol);
+  if (*fd_p >= 0)
     {
+      _dbus_verbose ("socket fd %d opened\n", *fd_p);
       return TRUE;
     }
   else
@@ -169,6 +172,57 @@ _dbus_write_socket (int               fd,
   return _dbus_write (fd, buffer, start, len);
 }
 
+/**
+ * write data to a pipe.
+ *
+ * @param pipe the pipe instance
+ * @param buffer the buffer to write data from
+ * @param start the first byte in the buffer to write
+ * @param len the number of bytes to try to write
+ * @param error error return
+ * @returns the number of bytes written or -1 on error
+ */
+int
+_dbus_pipe_write (DBusPipe         *pipe,
+                  const DBusString *buffer,
+                  int               start,
+                  int               len,
+                  DBusError        *error)
+{
+  int written;
+  
+  written = _dbus_write (pipe->fd_or_handle, buffer, start, len);
+  if (written < 0)
+    {
+      dbus_set_error (error, DBUS_ERROR_FAILED,
+                      "Writing to pipe: %s\n",
+                      _dbus_strerror (errno));
+    }
+  return written;
+}
+
+/**
+ * close a pipe.
+ *
+ * @param pipe the pipe instance
+ * @param error return location for an error
+ * @returns #FALSE if error is set
+ */
+int
+_dbus_pipe_close  (DBusPipe         *pipe,
+                   DBusError        *error)
+{
+  if (_dbus_close (pipe->fd_or_handle, error) < 0)
+    {
+      return -1;
+    }
+  else
+    {
+      _dbus_pipe_invalidate (pipe);
+      return 0;
+    }
+}
+
 /**
  * Like _dbus_write_two() but only works on sockets and is thus
  * available on Windows.
@@ -754,27 +808,28 @@ _dbus_connect_tcp_socket (const char     *host,
 }
 
 /**
- * Creates a socket and binds it to the given path,
- * then listens on the socket. The socket is
- * set to be nonblocking. 
+ * Creates a socket and binds it to the given path, then listens on
+ * the socket. The socket is set to be nonblocking.  In case of port=0
+ * a random free port is used and returned in the port parameter.
+ * If inaddr_any is specified, the hostname is ignored.
  *
  * @param host the host name to listen on
- * @param port the prot to listen on
+ * @param port the prot to listen on, if zero a free port will be used
+ * @param inaddr_any TRUE to listen on all local interfaces instead of on the host name
  * @param error return location for errors
  * @returns the listening file descriptor or -1 on error
  */
 int
 _dbus_listen_tcp_socket (const char     *host,
-                         dbus_uint32_t   port,
+                         dbus_uint32_t  *port,
+                         dbus_bool_t     inaddr_any,
                          DBusError      *error)
 {
   int listen_fd;
   struct sockaddr_in addr;
-  struct hostent *he;
-  struct in_addr *haddr;
+  socklen_t len = (socklen_t) sizeof (struct sockaddr);
 
-  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
-  
+  _DBUS_ASSERT_ERROR_IS_CLEAR (error);  
   
   if (!_dbus_open_tcp_socket (&listen_fd, error))
     {
@@ -783,29 +838,41 @@ _dbus_listen_tcp_socket (const char     *host,
     }
   _DBUS_ASSERT_ERROR_IS_CLEAR(error);
 
-  he = gethostbyname (host);
-  if (he == NULL) 
+  _DBUS_ZERO (addr);
+  
+  if (inaddr_any)
     {
-      dbus_set_error (error,
-                      _dbus_error_from_errno (errno),
-                      "Failed to lookup hostname: %s",
-                      host);
-      _dbus_close (listen_fd, NULL);
-      return -1;
+      addr.sin_addr.s_addr = INADDR_ANY;
+    }
+  else
+    {
+      struct hostent *he;
+      struct in_addr *haddr;      
+
+      he = gethostbyname (host);
+      if (he == NULL) 
+        {
+          dbus_set_error (error,
+                          _dbus_error_from_errno (errno),
+                          "Failed to lookup hostname: %s",
+                          host);
+          _dbus_close (listen_fd, NULL);
+          return -1;
+        }
+  
+      haddr = ((struct in_addr *) (he->h_addr_list)[0]);
+      
+      memcpy (&addr.sin_addr, haddr, sizeof (struct in_addr));
     }
   
-  haddr = ((struct in_addr *) (he->h_addr_list)[0]);
-
-  _DBUS_ZERO (addr);
-  memcpy (&addr.sin_addr, haddr, sizeof (struct in_addr));
   addr.sin_family = AF_INET;
-  addr.sin_port = htons (port);
+  addr.sin_port = htons (*port);
 
   if (bind (listen_fd, (struct sockaddr*) &addr, sizeof (struct sockaddr)))
     {
       dbus_set_error (error, _dbus_error_from_errno (errno),
                       "Failed to bind socket \"%s:%d\": %s",
-                      host, port, _dbus_strerror (errno));
+                      host, *port, _dbus_strerror (errno));
       _dbus_close (listen_fd, NULL);
       return -1;
     }
@@ -814,11 +881,14 @@ _dbus_listen_tcp_socket (const char     *host,
     {
       dbus_set_error (error, _dbus_error_from_errno (errno),  
                       "Failed to listen on socket \"%s:%d\": %s",
-                      host, port, _dbus_strerror (errno));
+                      host, *port, _dbus_strerror (errno));
       _dbus_close (listen_fd, NULL);
       return -1;
     }
 
+  getsockname(listen_fd, (struct sockaddr*) &addr, &len);
+  *port = (dbus_uint32_t) ntohs(addr.sin_port);
+
   if (!_dbus_set_fd_nonblocking (listen_fd, error))
     {
       _dbus_close (listen_fd, NULL);
@@ -892,11 +962,14 @@ write_credentials_byte (int             server_fd,
 
 /**
  * Reads a single byte which must be nul (an error occurs otherwise),
- * and reads unix credentials if available. Fills in pid/uid/gid with
- * -1 if no credentials are available. Return value indicates whether
- * a byte was read, not whether we got valid credentials. On some
- * systems, such as Linux, reading/writing the byte isn't actually
- * required, but we do it anyway just to avoid multiple codepaths.
+ * and reads unix credentials if available. Clears the credentials
+ * object, then adds pid/uid if available, so any previous credentials
+ * stored in the object are lost.
+ *
+ * Return value indicates whether a byte was read, not whether
+ * we got valid credentials. On some systems, such as Linux,
+ * reading/writing the byte isn't actually required, but we do it
+ * anyway just to avoid multiple codepaths.
  * 
  * Fails if no byte is available, so you must select() first.
  *
@@ -904,29 +977,35 @@ write_credentials_byte (int             server_fd,
  * use sendmsg()/recvmsg() to transmit credentials.
  *
  * @param client_fd the client file descriptor
- * @param credentials struct to fill with credentials of client
+ * @param credentials object to add client credentials to
  * @param error location to store error code
  * @returns #TRUE on success
  */
 dbus_bool_t
-_dbus_read_credentials_unix_socket  (int              client_fd,
-                                     DBusCredentials *credentials,
-                                     DBusError       *error)
+_dbus_read_credentials_socket  (int              client_fd,
+                                DBusCredentials *credentials,
+                                DBusError       *error)
 {
   struct msghdr msg;
   struct iovec iov;
   char buf;
-
+  dbus_uid_t uid_read;
+  dbus_pid_t pid_read;
+  int bytes_read;
+  
+  uid_read = DBUS_UID_UNSET;
+  pid_read = DBUS_PID_UNSET;
+  
 #ifdef HAVE_CMSGCRED 
   struct {
-	  struct cmsghdr hdr;
-	  struct cmsgcred cred;
+    struct cmsghdr hdr;
+    struct cmsgcred cred;
   } cmsg;
 
 #elif defined(LOCAL_CREDS)
   struct {
-	  struct cmsghdr hdr;
-	  struct sockcred cred;
+    struct cmsghdr hdr;
+    struct sockcred cred;
   } cmsg;
 #endif
 
@@ -936,9 +1015,9 @@ _dbus_read_credentials_unix_socket  (int              client_fd,
    * we need these assertions to fail as soon as we're wrong about
    * it so we can do the porting fixups
    */
-  _dbus_assert (sizeof (pid_t) <= sizeof (credentials->pid));
-  _dbus_assert (sizeof (uid_t) <= sizeof (credentials->uid));
-  _dbus_assert (sizeof (gid_t) <= sizeof (credentials->gid));
+  _dbus_assert (sizeof (pid_t) <= sizeof (dbus_pid_t));
+  _dbus_assert (sizeof (uid_t) <= sizeof (dbus_uid_t));
+  _dbus_assert (sizeof (gid_t) <= sizeof (dbus_gid_t));
 
   _dbus_credentials_clear (credentials);
 
@@ -962,18 +1041,33 @@ _dbus_read_credentials_unix_socket  (int              client_fd,
 #endif
 
  again:
-  if (recvmsg (client_fd, &msg, 0) < 0)
+  bytes_read = recvmsg (client_fd, &msg, 0);
+
+  if (bytes_read < 0)
     {
       if (errno == EINTR)
 	goto again;
 
+      /* EAGAIN or EWOULDBLOCK would be unexpected here since we would
+       * normally only call read_credentials if the socket was ready
+       * for reading
+       */
+      
       dbus_set_error (error, _dbus_error_from_errno (errno),
                       "Failed to read credentials byte: %s",
                       _dbus_strerror (errno));
       return FALSE;
     }
-
-  if (buf != '\0')
+  else if (bytes_read == 0)
+    {
+      /* this should not happen unless we are using recvmsg wrong,
+       * so is essentially here for paranoia
+       */
+      dbus_set_error (error, DBUS_ERROR_FAILED,
+                      "Failed to read credentials byte (zero-length read)");
+      return FALSE;
+    }
+  else if (buf != '\0')
     {
       dbus_set_error (error, DBUS_ERROR_FAILED,
                       "Credentials byte was not nul");
@@ -999,9 +1093,8 @@ _dbus_read_credentials_unix_socket  (int              client_fd,
     if (getsockopt (client_fd, SOL_SOCKET, SO_PEERCRED, &cr, &cr_len) == 0 &&
 	cr_len == sizeof (cr))
       {
-	credentials->pid = cr.pid;
-	credentials->uid = cr.uid;
-	credentials->gid = cr.gid;
+	pid_read = cr.pid;
+	uid_read = cr.uid;
       }
     else
       {
@@ -1009,13 +1102,11 @@ _dbus_read_credentials_unix_socket  (int              client_fd,
 		       cr_len, (int) sizeof (cr), _dbus_strerror (errno));
       }
 #elif defined(HAVE_CMSGCRED)
-    credentials->pid = cmsg.cred.cmcred_pid;
-    credentials->uid = cmsg.cred.cmcred_euid;
-    credentials->gid = cmsg.cred.cmcred_groups[0];
+    pid_read = cmsg.cred.cmcred_pid;
+    uid_read = cmsg.cred.cmcred_euid;
 #elif defined(LOCAL_CREDS)
-    credentials->pid = DBUS_PID_UNSET;
-    credentials->uid = cmsg.cred.sc_uid;
-    credentials->gid = cmsg.cred.sc_gid;
+    pid_read = DBUS_PID_UNSET;
+    uid_read = cmsg.cred.sc_uid;
     /* Since we have already got the credentials from this socket, we can
      * disable its LOCAL_CREDS flag if it was ever set. */
     _dbus_set_local_creds (client_fd, FALSE);
@@ -1024,8 +1115,7 @@ _dbus_read_credentials_unix_socket  (int              client_fd,
     gid_t egid;
     if (getpeereid (client_fd, &euid, &egid) == 0)
       {
-        credentials->uid = euid;
-        credentials->gid = egid;
+        uid_read = euid;
       }
     else
       {
@@ -1035,9 +1125,8 @@ _dbus_read_credentials_unix_socket  (int              client_fd,
     ucred_t * ucred = NULL;
     if (getpeerucred (client_fd, &ucred) == 0)
       {
-        credentials->pid = ucred_getpid (ucred);
-        credentials->uid = ucred_geteuid (ucred);
-        credentials->gid = ucred_getegid (ucred);
+        pid_read = ucred_getpid (ucred);
+        uid_read = ucred_geteuid (ucred);
       }
     else
       {
@@ -1053,11 +1142,28 @@ _dbus_read_credentials_unix_socket  (int              client_fd,
   _dbus_verbose ("Credentials:"
                  "  pid "DBUS_PID_FORMAT
                  "  uid "DBUS_UID_FORMAT
-                 "  gid "DBUS_GID_FORMAT"\n",
-		 credentials->pid,
-		 credentials->uid,
-		 credentials->gid);
-    
+                 "\n",
+		 pid_read,
+		 uid_read);
+
+  if (pid_read != DBUS_PID_UNSET)
+    {
+      if (!_dbus_credentials_add_unix_pid (credentials, pid_read))
+        {
+          _DBUS_SET_OOM (error);
+          return FALSE;
+        }
+    }
+
+  if (uid_read != DBUS_UID_UNSET)
+    {
+      if (!_dbus_credentials_add_unix_uid (credentials, uid_read))
+        {
+          _DBUS_SET_OOM (error);
+          return FALSE;
+        }
+    }
+  
   return TRUE;
 }
 
@@ -1079,8 +1185,8 @@ _dbus_read_credentials_unix_socket  (int              client_fd,
  * @returns #TRUE if the byte was sent
  */
 dbus_bool_t
-_dbus_send_credentials_unix_socket  (int              server_fd,
-                                     DBusError       *error)
+_dbus_send_credentials_socket  (int              server_fd,
+                                DBusError       *error)
 {
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   
@@ -1114,6 +1220,8 @@ _dbus_accept  (int listen_fd)
       if (errno == EINTR)
         goto retry;
     }
+
+  _dbus_verbose ("client fd %d accepted\n", client_fd);
   
   return client_fd;
 }
@@ -1389,31 +1497,54 @@ _dbus_user_info_fill_uid (DBusUserInfo *info,
 }
 
 /**
- * Gets the credentials of the current process.
+ * Adds the credentials of the current process to the
+ * passed-in credentials object.
  *
- * @param credentials credentials to fill in.
+ * @param credentials credentials to add to
+ * @returns #FALSE if no memory; does not properly roll back on failure, so only some credentials may have been added
  */
-void
-_dbus_credentials_from_current_process (DBusCredentials *credentials)
+dbus_bool_t
+_dbus_credentials_add_from_current_process (DBusCredentials *credentials)
 {
   /* The POSIX spec certainly doesn't promise this, but
    * we need these assertions to fail as soon as we're wrong about
    * it so we can do the porting fixups
    */
-  _dbus_assert (sizeof (pid_t) <= sizeof (credentials->pid));
-  _dbus_assert (sizeof (uid_t) <= sizeof (credentials->uid));
-  _dbus_assert (sizeof (gid_t) <= sizeof (credentials->gid));
-  
-  credentials->pid = getpid ();
-  credentials->uid = getuid ();
-  credentials->gid = getgid ();
+  _dbus_assert (sizeof (pid_t) <= sizeof (dbus_pid_t));
+  _dbus_assert (sizeof (uid_t) <= sizeof (dbus_uid_t));
+  _dbus_assert (sizeof (gid_t) <= sizeof (dbus_gid_t));
+
+  if (!_dbus_credentials_add_unix_pid(credentials, _dbus_getpid()))
+    return FALSE;
+  if (!_dbus_credentials_add_unix_uid(credentials, _dbus_getuid()))
+    return FALSE;
+
+  return TRUE;
+}
+
+/**
+ * Append to the string the identity we would like to have when we
+ * authenticate, on UNIX this is the current process UID and on
+ * Windows something else, probably a Windows SID string.  No escaping
+ * is required, that is done in dbus-auth.c. The username here
+ * need not be anything human-readable, it can be the machine-readable
+ * form i.e. a user id.
+ * 
+ * @param str the string to append to
+ * @returns #FALSE on no memory
+ */
+dbus_bool_t
+_dbus_append_user_from_current_process (DBusString *str)
+{
+  return _dbus_string_append_uint (str,
+                                   _dbus_getuid ());
 }
 
 /**
  * Gets our process ID
  * @returns process ID
  */
-unsigned long
+dbus_pid_t
 _dbus_getpid (void)
 {
   return getpid ();
@@ -1428,6 +1559,124 @@ _dbus_getuid (void)
   return getuid ();
 }
 
+/**
+ * The only reason this is separate from _dbus_getpid() is to allow it
+ * on Windows for logging but not for other purposes.
+ * 
+ * @returns process ID to put in log messages
+ */
+unsigned long
+_dbus_pid_for_log (void)
+{
+  return getpid ();
+}
+
+/**
+ * Gets a UID from a UID string.
+ *
+ * @param uid_str the UID in string form
+ * @param uid UID to fill in
+ * @returns #TRUE if successfully filled in UID
+ */
+dbus_bool_t
+_dbus_parse_uid (const DBusString      *uid_str,
+                 dbus_uid_t            *uid)
+{
+  int end;
+  long val;
+  
+  if (_dbus_string_get_length (uid_str) == 0)
+    {
+      _dbus_verbose ("UID string was zero length\n");
+      return FALSE;
+    }
+
+  val = -1;
+  end = 0;
+  if (!_dbus_string_parse_int (uid_str, 0, &val,
+                               &end))
+    {
+      _dbus_verbose ("could not parse string as a UID\n");
+      return FALSE;
+    }
+  
+  if (end != _dbus_string_get_length (uid_str))
+    {
+      _dbus_verbose ("string contained trailing stuff after UID\n");
+      return FALSE;
+    }
+
+  *uid = val;
+
+  return TRUE;
+}
+
+
+_DBUS_DEFINE_GLOBAL_LOCK (atomic);
+
+#ifdef DBUS_USE_ATOMIC_INT_486
+/* Taken from CVS version 1.7 of glibc's sysdeps/i386/i486/atomicity.h */
+/* Since the asm stuff here is gcc-specific we go ahead and use "inline" also */
+static inline dbus_int32_t
+atomic_exchange_and_add (DBusAtomic            *atomic,
+                         volatile dbus_int32_t  val)
+{
+  register dbus_int32_t result;
+
+  __asm__ __volatile__ ("lock; xaddl %0,%1"
+                        : "=r" (result), "=m" (atomic->value)
+			: "0" (val), "m" (atomic->value));
+  return result;
+}
+#endif
+
+/**
+ * Atomically increments an integer
+ *
+ * @param atomic pointer to the integer to increment
+ * @returns the value before incrementing
+ *
+ * @todo implement arch-specific faster atomic ops
+ */
+dbus_int32_t
+_dbus_atomic_inc (DBusAtomic *atomic)
+{
+#ifdef DBUS_USE_ATOMIC_INT_486
+  return atomic_exchange_and_add (atomic, 1);
+#else
+  dbus_int32_t res;
+  _DBUS_LOCK (atomic);
+  res = atomic->value;
+  atomic->value += 1;
+  _DBUS_UNLOCK (atomic);
+  return res;
+#endif
+}
+
+/**
+ * Atomically decrement an integer
+ *
+ * @param atomic pointer to the integer to decrement
+ * @returns the value before decrementing
+ *
+ * @todo implement arch-specific faster atomic ops
+ */
+dbus_int32_t
+_dbus_atomic_dec (DBusAtomic *atomic)
+{
+#ifdef DBUS_USE_ATOMIC_INT_486
+  return atomic_exchange_and_add (atomic, -1);
+#else
+  dbus_int32_t res;
+  
+  _DBUS_LOCK (atomic);
+  res = atomic->value;
+  atomic->value -= 1;
+  _DBUS_UNLOCK (atomic);
+  return res;
+#endif
+}
+
 #ifdef DBUS_BUILD_TESTS
 /** Gets our GID
  * @returns process GID
@@ -1595,6 +1844,8 @@ _dbus_file_get_contents (DBusString       *str,
       return FALSE;
     }
 
+  _dbus_verbose ("file fd %d opened\n", fd);
+  
   if (fstat (fd, &sb) < 0)
     {
       dbus_set_error (error, _dbus_error_from_errno (errno),
@@ -1736,6 +1987,8 @@ _dbus_string_save_to_file (const DBusString *str,
       goto out;
     }
 
+  _dbus_verbose ("tmp file fd %d opened\n", fd);
+  
   need_unlink = TRUE;
   
   total = 0;
@@ -1861,6 +2114,8 @@ _dbus_create_file_exclusively (const DBusString *filename,
       return FALSE;
     }
 
+  _dbus_verbose ("exclusive file fd %d opened\n", fd);
+  
   if (!_dbus_close (fd, NULL))
     {
       dbus_set_error (error,
@@ -2059,6 +2314,8 @@ _dbus_generate_random_bytes (DBusString *str,
   if (fd < 0)
     return _dbus_generate_pseudorandom_bytes (str, n_bytes);
 
+  _dbus_verbose ("/dev/urandom fd %d opened\n", fd);
+  
   if (_dbus_read (fd, str, n_bytes) != n_bytes)
     {
       _dbus_close (fd, NULL);
@@ -2087,7 +2344,8 @@ _dbus_exit (int code)
 
 /**
  * A wrapper around strerror() because some platforms
- * may be lame and not have strerror().
+ * may be lame and not have strerror(). Also, never
+ * returns NULL.
  *
  * @param error_number errno.
  * @returns error description.
@@ -2369,8 +2627,9 @@ dbus_bool_t
 _dbus_get_autolaunch_address (DBusString *address,
                               DBusError  *error)
 {
-  static char *argv[5];
-  int address_pipe[2];
+  static char *argv[6];
+  int address_pipe[2] = { -1, -1 };
+  int errors_pipe[2] = { -1, -1 };
   pid_t pid;
   int ret;
   int status;
@@ -2399,6 +2658,8 @@ _dbus_get_autolaunch_address (DBusString *address,
   ++i;
   argv[i] = "--binary-syntax";
   ++i;
+  argv[i] = "--close-stderr";
+  ++i;
   argv[i] = NULL;
   ++i;
 
@@ -2417,6 +2678,15 @@ _dbus_get_autolaunch_address (DBusString *address,
                      _dbus_strerror (errno));
       goto out;
     }
+  if (pipe (errors_pipe) < 0)
+    {
+      dbus_set_error (error, _dbus_error_from_errno (errno),
+                      "Failed to create a pipe: %s",
+                      _dbus_strerror (errno));
+      _dbus_verbose ("Failed to create a pipe to call dbus-launch: %s\n",
+                     _dbus_strerror (errno));
+      goto out;
+    }
 
   pid = fork ();
   if (pid < 0)
@@ -2437,8 +2707,11 @@ _dbus_get_autolaunch_address (DBusString *address,
         /* huh?! can't open /dev/null? */
         _exit (1);
 
+      _dbus_verbose ("/dev/null fd %d opened\n", fd);
+      
       /* set-up stdXXX */
       close (address_pipe[READ_END]);
+      close (errors_pipe[READ_END]);
       close (0);                /* close stdin */
       close (1);                /* close stdout */
       close (2);                /* close stderr */
@@ -2447,11 +2720,12 @@ _dbus_get_autolaunch_address (DBusString *address,
         _exit (1);
       if (dup2 (address_pipe[WRITE_END], 1) == -1)
         _exit (1);
-      if (dup2 (fd, 2) == -1)
+      if (dup2 (errors_pipe[WRITE_END], 2) == -1)
         _exit (1);
 
       close (fd);
       close (address_pipe[WRITE_END]);
+      close (errors_pipe[WRITE_END]);
 
       execv (DBUS_BINDIR "/dbus-launch", argv);
 
@@ -2464,6 +2738,10 @@ _dbus_get_autolaunch_address (DBusString *address,
 
   /* parent process */
   close (address_pipe[WRITE_END]);
+  close (errors_pipe[WRITE_END]);
+  address_pipe[WRITE_END] = -1;
+  errors_pipe[WRITE_END] = -1;
+
   ret = 0;
   do 
     {
@@ -2484,9 +2762,23 @@ _dbus_get_autolaunch_address (DBusString *address,
       _dbus_string_get_length (address) == orig_len)
     {
       /* The process ended with error */
+      DBusString error_message;
+      _dbus_string_init (&error_message);
+      ret = 0;
+      do
+	{
+	  ret = _dbus_read (errors_pipe[READ_END], &error_message, 1024);
+	}
+      while (ret > 0);
+
       _dbus_string_set_length (address, orig_len);
-      dbus_set_error (error, DBUS_ERROR_SPAWN_EXEC_FAILED,
-                      "Failed to execute dbus-launch to autolaunch D-Bus session");
+      if (_dbus_string_get_length (&error_message) > 0)
+	dbus_set_error (error, DBUS_ERROR_SPAWN_EXEC_FAILED,
+			"dbus-launch failed to autolaunch D-Bus session: %s",
+			_dbus_string_get_data (&error_message));
+      else
+	dbus_set_error (error, DBUS_ERROR_SPAWN_EXEC_FAILED,
+			"Failed to execute dbus-launch to autolaunch D-Bus session");
       goto out;
     }
 
@@ -2497,7 +2789,16 @@ _dbus_get_autolaunch_address (DBusString *address,
     _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   else
     _DBUS_ASSERT_ERROR_IS_SET (error);
-  
+
+  if (address_pipe[0] != -1)
+    close (address_pipe[0]);
+  if (address_pipe[1] != -1)
+    close (address_pipe[1]);
+  if (errors_pipe[0] != -1)
+    close (errors_pipe[0]);
+  if (errors_pipe[1] != -1)
+    close (errors_pipe[1]);
+
   _dbus_string_free (&uuid);
   return retval;
 }
@@ -2532,120 +2833,6 @@ _dbus_read_local_machine_uuid (DBusGUID   *machine_id,
 
 #define DBUS_UNIX_STANDARD_SESSION_SERVICEDIR "/dbus-1/services"
 
-static dbus_bool_t
-split_paths_and_append (DBusString *dirs, 
-                        const char *suffix, 
-                        DBusList **dir_list)
-{
-  /* split on colon (:) */
-   int start;
-   int i;
-   int len;
-   char *cpath;
-   const DBusString file_suffix;
-
-   start = 0;
-   i = 0;
-
-   _dbus_string_init_const (&file_suffix, suffix);
-
-   len = _dbus_string_get_length (dirs);
-
-   while (_dbus_string_find (dirs, start, ":", &i))
-     {
-       DBusString path;
-
-       if (!_dbus_string_init (&path))
-          goto oom;
-
-       if (!_dbus_string_copy_len (dirs,
-                                   start,
-                                   i - start,
-                                   &path,
-                                   0))
-          {
-            _dbus_string_free (&path);
-            goto oom;
-          }
-
-        _dbus_string_chop_white (&path);
-
-        /* check for an empty path */
-        if (_dbus_string_get_length (&path) == 0)
-          goto next;
-
-        if (!_dbus_concat_dir_and_file (&path,
-                                        &file_suffix))
-          {
-            _dbus_string_free (&path);
-            goto oom;
-          }
-
-        if (!_dbus_string_copy_data(&path, &cpath))
-          {
-            _dbus_string_free (&path);
-            goto oom;
-          }
-
-        if (!_dbus_list_append (dir_list, cpath))
-          {
-            _dbus_string_free (&path);              
-            dbus_free (cpath);
-            goto oom;
-          }
-
-       next:
-        _dbus_string_free (&path);
-        start = i + 1;
-    } 
-      
-  if (start != len)
-    { 
-      DBusString path;
-
-      if (!_dbus_string_init (&path))
-        goto oom;
-
-      if (!_dbus_string_copy_len (dirs,
-                                  start,
-                                  len - start,
-                                  &path,
-                                  0))
-        {
-          _dbus_string_free (&path);
-          goto oom;
-        }
-
-      if (!_dbus_concat_dir_and_file (&path,
-                                      &file_suffix))
-        {
-          _dbus_string_free (&path);
-          goto oom;
-        }
-
-      if (!_dbus_string_copy_data(&path, &cpath))
-        {
-          _dbus_string_free (&path);
-          goto oom;
-        }
-
-      if (!_dbus_list_append (dir_list, cpath))
-        {
-          _dbus_string_free (&path);              
-          dbus_free (cpath);
-          goto oom;
-        }
-
-      _dbus_string_free (&path); 
-    }
-
-  return TRUE;
-
- oom:
-  _dbus_list_foreach (dir_list, (DBusForeachFunction)dbus_free, NULL); 
-  _dbus_list_clear (dir_list);
-  return FALSE;
-}
 
 /**
  * Returns the standard directories for a session bus to look for service 
@@ -2708,7 +2895,7 @@ _dbus_get_standard_session_servicedirs (DBusList **dirs)
   else
     {
       const DBusString *homedir;
-      const DBusString local_share;
+      DBusString local_share;
 
       if (!_dbus_homedir_from_current_process (&homedir))
         goto oom;
@@ -2721,9 +2908,9 @@ _dbus_get_standard_session_servicedirs (DBusList **dirs)
         goto oom;
     }
 
-  if (!split_paths_and_append (&servicedir_path, 
-                               DBUS_UNIX_STANDARD_SESSION_SERVICEDIR, 
-                               dirs))
+  if (!_dbus_split_paths_and_append (&servicedir_path, 
+                                     DBUS_UNIX_STANDARD_SESSION_SERVICEDIR, 
+                                     dirs))
     goto oom;
 
   _dbus_string_free (&servicedir_path);  
@@ -2734,4 +2921,133 @@ _dbus_get_standard_session_servicedirs (DBusList **dirs)
   return FALSE;
 }
 
+/**
+ * Append the absolute path of the system.conf file
+ * (there is no system bus on Windows so this can just
+ * return FALSE and print a warning or something)
+ * 
+ * @param str the string to append to
+ * @returns #FALSE if no memory
+ */
+dbus_bool_t
+_dbus_append_system_config_file (DBusString *str)
+{
+  return _dbus_string_append (str, DBUS_SYSTEM_CONFIG_FILE);
+}
+
+/**
+ * Append the absolute path of the session.conf file.
+ * 
+ * @param str the string to append to
+ * @returns #FALSE if no memory
+ */
+dbus_bool_t
+_dbus_append_session_config_file (DBusString *str)
+{
+  return _dbus_string_append (str, DBUS_SESSION_CONFIG_FILE);
+}
+
+/**
+ * Called when the bus daemon is signaled to reload its configuration; any
+ * caches should be nuked. Of course any caches that need explicit reload
+ * are probably broken, but c'est la vie.
+ *
+ * 
+ */
+void
+_dbus_flush_caches (void)
+{
+  _dbus_user_database_flush_system ();
+}
+
+/**
+ * Appends the directory in which a keyring for the given credentials
+ * should be stored.  The credentials should have either a Windows or
+ * UNIX user in them.  The directory should be an absolute path.
+ *
+ * On UNIX the directory is ~/.dbus-keyrings while on Windows it should probably
+ * be something else, since the dotfile convention is not normal on Windows.
+ * 
+ * @param directory string to append directory to
+ * @param credentials credentials the directory should be for
+ *  
+ * @returns #FALSE on no memory
+ */
+dbus_bool_t
+_dbus_append_keyring_directory_for_credentials (DBusString      *directory,
+                                                DBusCredentials *credentials)
+{
+  DBusString homedir;
+  DBusString dotdir;
+  dbus_uid_t uid;
+  
+  _dbus_assert (credentials != NULL);
+  _dbus_assert (!_dbus_credentials_are_anonymous (credentials));
+  
+  if (!_dbus_string_init (&homedir))
+    return FALSE;
+
+  uid = _dbus_credentials_get_unix_uid (credentials);
+  _dbus_assert (uid != DBUS_UID_UNSET);
+
+  if (!_dbus_homedir_from_uid (uid, &homedir))
+    goto failed;
+  
+#ifdef DBUS_BUILD_TESTS
+  {
+    const char *override;
+    
+    override = _dbus_getenv ("DBUS_TEST_HOMEDIR");
+    if (override != NULL && *override != '\0')
+      {
+        _dbus_string_set_length (&homedir, 0);
+        if (!_dbus_string_append (&homedir, override))
+          goto failed;
+
+        _dbus_verbose ("Using fake homedir for testing: %s\n",
+                       _dbus_string_get_const_data (&homedir));
+      }
+    else
+      {
+        static dbus_bool_t already_warned = FALSE;
+        if (!already_warned)
+          {
+            _dbus_warn ("Using your real home directory for testing, set DBUS_TEST_HOMEDIR to avoid\n");
+            already_warned = TRUE;
+          }
+      }
+  }
+#endif
+
+  _dbus_string_init_const (&dotdir, ".dbus-keyrings");
+  if (!_dbus_concat_dir_and_file (&homedir,
+                                  &dotdir))
+    goto failed;
+  
+  if (!_dbus_string_copy (&homedir, 0,
+                          directory, _dbus_string_get_length (directory))) {
+    goto failed;
+  }
+
+  _dbus_string_free (&homedir);
+  return TRUE;
+  
+ failed: 
+  _dbus_string_free (&homedir);
+  return FALSE;
+}
+
+
+/**
+ * See if errno is EAGAIN or EWOULDBLOCK (this has to be done differently
+ * for Winsock so is abstracted)
+ *
+ * @returns #TRUE if errno == EAGAIN or errno == EWOULDBLOCK
+ */
+dbus_bool_t
+_dbus_get_is_errno_eagain_or_ewouldblock (void)
+{
+  return errno == EAGAIN || errno == EWOULDBLOCK;
+}
+
 /* tests in dbus-sysdeps-util.c */
