@@ -1,5 +1,5 @@
 /* -*- mode: C; c-file-style: "gnu" -*- */
-/* dbus-transport-protected.h Used by subclasses of DBusTransport object (internal to D-BUS implementation)
+/* dbus-transport-protected.h Used by subclasses of DBusTransport object (internal to D-Bus implementation)
  *
  * Copyright (C) 2002, 2004  Red Hat Inc.
  *
@@ -66,9 +66,9 @@ struct DBusTransportVTable
   void        (* live_messages_changed) (DBusTransport *transport);
   /**< Outstanding messages counter changed */
 
-  dbus_bool_t (* get_unix_fd) (DBusTransport *transport,
-                               int           *fd_p);
-  /**< Get UNIX file descriptor */
+  dbus_bool_t (* get_socket_fd) (DBusTransport *transport,
+                                 int           *fd_p);
+  /**< Get socket file descriptor */
 };
 
 /**
@@ -89,7 +89,7 @@ struct DBusTransport
 
   DBusAuth *auth;                             /**< Authentication conversation */
 
-  DBusCredentials credentials;                /**< Credentials of other end */  
+  DBusCredentials *credentials;               /**< Credentials of other end read from the socket */  
 
   long max_live_messages_size;                /**< Max total size of received messages. */
 
@@ -104,6 +104,11 @@ struct DBusTransport
   void *unix_user_data;                         /**< Data for unix_user_function */
   
   DBusFreeFunction free_unix_user_data;         /**< Function to free unix_user_data */
+
+  DBusAllowWindowsUserFunction windows_user_function; /**< Function for checking whether a user is authorized. */
+  void *windows_user_data;                            /**< Data for windows_user_function */
+  
+  DBusFreeFunction free_windows_user_data;            /**< Function to free windows_user_data */
   
   unsigned int disconnected : 1;              /**< #TRUE if we are disconnected. */
   unsigned int authenticated : 1;             /**< Cache of auth state; use _dbus_transport_get_is_authenticated() to query value */
@@ -111,6 +116,7 @@ struct DBusTransport
   unsigned int receive_credentials_pending : 1; /**< #TRUE if we need to receive credentials */
   unsigned int is_server : 1;                 /**< #TRUE if on the server side */
   unsigned int unused_bytes_recovered : 1;    /**< #TRUE if we've recovered unused bytes from auth */
+  unsigned int allow_anonymous : 1;           /**< #TRUE if an anonymous client can connect */
 };
 
 dbus_bool_t _dbus_transport_init_base     (DBusTransport             *transport,
@@ -120,6 +126,17 @@ dbus_bool_t _dbus_transport_init_base     (DBusTransport             *transport,
 void        _dbus_transport_finalize_base (DBusTransport             *transport);
 
 
+typedef enum
+{
+  DBUS_TRANSPORT_OPEN_NOT_HANDLED,    /**< we aren't in charge of this address type */
+  DBUS_TRANSPORT_OPEN_OK,             /**< we set up the listen */
+  DBUS_TRANSPORT_OPEN_BAD_ADDRESS,    /**< malformed address */
+  DBUS_TRANSPORT_OPEN_DID_NOT_CONNECT /**< well-formed address but failed to set it up */
+} DBusTransportOpenResult;
+
+DBusTransportOpenResult _dbus_transport_open_platform_specific (DBusAddressEntry  *entry,
+                                                                DBusTransport    **transport_p,
+                                                                DBusError         *error);
 
 DBUS_END_DECLS
 
